#include <exec/types.h>
#include "showprioq.h"

ULONG last_red = 0, last_green = 0, last_blue = 0;
int Close_Threshold;

#define absdif(x,y) ((x>y) ? (x-y):(y-x))
#define Make_Colour(x, y, z) ((x & 0xF) << 8) + ((y & 0xF) << 4) + (z & 0xF)

#define extract_red(x) ((x & 0xF00) >> 8)
#define extract_green(x) ((x & 0x0F0) >> 4)
#define extract_blue(x) (x & 0x00F)

struct prioq_struct *Colour_q;
struct prioq_struct *pq_new();

extern UWORD ColourTbl[16];

reset_colours ()
  {
  last_red = last_green = last_blue = 0;
  }

closeness(x, y)
  int x, y;
  {
  int red, blue, green;

  red = absdif(extract_red(x), extract_red(y));
  blue = absdif (extract_blue(x), extract_blue(y));
  green = absdif(extract_green(x), extract_green(y));
  return (red+blue+green);
  }

start_recording_colours ()
  {
  Colour_q = pq_new (256, 4096);
  if (Colour_q == NULL)
    exit (0);

  reset_colours();
  }

record_colours (new_red, new_green, new_blue)
  int new_red, new_green, new_blue;
  {
  LONG delta_red, delta_green, delta_blue, match_quality;

  delta_red = absdif (new_red, last_red);
  delta_green = absdif (new_green, last_green);
  delta_blue = absdif (new_blue, last_blue);

  if (delta_red > delta_green)
    if (delta_red > delta_blue) {
      last_red = new_red;
      match_quality = delta_green + delta_blue;
      }
    else {
      last_blue = new_blue;
      match_quality = delta_green + delta_red;
      }
  else
    if (delta_green > delta_blue)
      {
      last_green = new_green;
      match_quality = delta_red + delta_blue;
      }
    else {
      last_blue = new_blue;
      match_quality = delta_green + delta_red;
      }

    if (match_quality != 0)
      pq_add (Colour_q, match_quality,
              Make_Colour (new_red, new_green, new_blue));
  }      

choose_palette()
  {
  int i, j, colour, min_distance, temp_distance;
  struct prioq_struct *post_pq;

  post_pq = pq_new (32, 4096);

  ColourTbl[0] = Make_Colour (0, 0, 0);
  for (i=1 ; i < 16 ;) {
    if (pq_get_highest_index (Colour_q) > pq_get_highest_index (post_pq))
      {
      colour = pq_get_highest_value (Colour_q);
      pq_delete_highest (Colour_q);

      min_distance = 255;
      for (j = 0 ; j < i ; j++)
       if ((temp_distance = closeness (ColourTbl[j], colour)) < min_distance)
         min_distance = temp_distance;

      if (min_distance < 5)
        pq_add (post_pq, min_distance, colour);
      else
       ColourTbl[i++] = colour;
      }
    else
      {
      ColourTbl[i++] = pq_get_highest_value (post_pq);
      pq_delete_highest (post_pq);
      }
    }
  pq_free (Colour_q);
  pq_free (post_pq);
  }

best_colour (new_red, new_blue, new_green)
  int new_red, new_blue, new_green;
  {
  int i, match_quality, best_match, colour,
      delta_red, delta_green, delta_blue, temp_match_quality;

  delta_red = absdif (new_red, last_red);
  delta_green = absdif (new_green, last_green);
  delta_blue = absdif (new_blue, last_blue);

  if (delta_red > delta_green)
    if (delta_red > delta_blue) {
      last_red = new_red;
      colour = 0x20 + new_red;
      match_quality = delta_green + delta_blue;
      }
    else {
      last_blue = new_blue;
      colour = 0x10 + new_blue;
      match_quality = delta_green + delta_red;
      }
  else
    if (delta_green > delta_blue)
      {
      last_green = new_green;
      colour = 0x30 + new_green;
      match_quality = delta_red + delta_blue;
      }
    else {
      last_blue = new_blue;
      colour = 0x10 + new_blue;
      match_quality = delta_green + delta_red;
      }

  if (match_quality != 0)
    for (i = 0 ; i < 16 ; i++)
      if ((temp_match_quality =
           closeness (ColourTbl[i],
            Make_Colour (new_red, new_green, new_blue)))
          < match_quality) {
        match_quality = temp_match_quality;
        colour = i;
        last_red = extract_red (ColourTbl[i]);
        last_green = extract_green (ColourTbl[i]);
        last_blue = extract_blue (ColourTbl[i]);
        }
  return (colour);
  }

